function [] = generate_loadfile_train;

% readmfcc
% filter_data

% Train features are generated and they will be saved at /Data/train_feats.mat

%%% Start Initial definitions %%%
global mypath;
global myfeatspath;

frame_rate = 100; % Check this variable.
mfcext = '.mfc';  % Extention of feature filename.

[Sp us ue Spid] = textread(strcat(mypath, 'Sessions/train.lst'),'%s %f %f %s');

rem_sil = 1; % To remove silence.
cSp = '';    % Current Speaker
spid = 0;
rev = 0;
spid_cnt = 0;
%%% End Initial definitions %%%

%%% Start building train files %%%
for cnt = 1:length(Sp);
    [fname, fext] = strread(Sp{cnt}, '%s%s', 'delimiter', '/');

    %%% If current filename is not current speaker then it indicates a new
    %%% speaker. So increase speaker count. %%%
    if(~strcmp(fname{1}, cSp))
        spid = spid + 1;
        Speakers{spid} = fname{1};
        cSp = fname{1};
        % spid serves as indicator for cSp (current speaker)
        spid_cnt = 0;
    end

    % Create myRpath for reading mfcc files.
    myRpath = strcat(myfeatspath, 'Train/',Sp{cnt});

    %%% If MFCC file does not exist then print error message.
    if(~exist(myRpath))
        err_str = sprintf('%s does not exist', myRpath);
        error(err_str);
    end


    % Read mfcc file
    data = readmfcc(myRpath);
    % Convert time in seconds to time in frame no.
    % Doing read operations multiple times for a repeated file.
    sfid = max(1, 1 + floor(us(cnt) * frame_rate));
    efid = min(ceil(ue(cnt) * frame_rate), size(data, 2));
    efid = max(1, efid);
    
    if(sfid>efid)
        error('Error, start is later than end frame');
        fname
    end
    data = data(:, sfid:efid);
    % data = filter_data(data, rev);

    spid_cnt = spid_cnt + 1;
    Feat{spid}{spid_cnt} = data;
end

% Save operation.
savefilen = strcat(mypath, 'Data/train_feats');
TrainFeat = Feat; clear Feat;
save(savefilen, 'TrainFeat', 'Speakers');