function [data, Tloglike] = mix_gau_N_diag_var(x,Ngau, startM)

% Dimension of x is = Nvec * Nobservations.
% Dimension of mean is = Nvec * Ngau.
% Dimension of Var is  = Nvec * Nvec * Ngau.

% Dependencies :
% mean_var
% gauss_dist

global var_floor;
var_floor = 1e-4;

if(size(x,2) < size(x,1))
    error('x should row Matrix');
end

% If 1 gaussian, return the values direclty.
if(Ngau == 1)
    data.prob = 1;
    [data.means, data.vars]  = mean_var(x);
    Tloglike = 1;
    return;
end
% [ids, mymeans] = kmeans(x', Ngau, 'Maxiter', 65, 'EmptyAction','singleton','display','off');
[ids, mymeans] = kmeans(x(:,1:2:end)', Ngau, 'Maxiter', 65, 'EmptyAction','singleton','display','off'); % 5/27/08

data.means = mymeans';
data.prob  = ones(Ngau,1)/Ngau;
initprob = repmat(1/Ngau,1,size(x,2));
initvar = expect_var(x, initprob, data.means(:,1))/Ngau;

for ngau = 1:Ngau
    data.vars(:,:,ngau) = initvar;
end

I = 40;
loglike = [];
diffperunitlike = 5 * 1e-4; %Convergence should depend on amount of data.
like = [];

%%%%%%%Initialization of code.%%%%%%%%%
if(isempty(like))
    % PY_x is of size Ngau * length(x).
    for ngau = 1:Ngau
        like(ngau, :)     = data.prob(ngau) * gauss_dist(x, data.means(:, ngau), data.vars(:,:,ngau));
    end
end
PY_x = like;
[PY_x] = norm_prob(PY_x);
sum_like = sum(like,1);

% Some issues with 0, if some probabilities are zero then log zero issue,
% so remove count of such probabilities of such events for now.

sum_like(sum_like == 0) = [];
loglike(1) = sum(log(sum_like));
%%%%%%%Finish Initialization.%%%%%%%%%

for iter = 2:I
    oldmean = data.means;
    oldvar  = data.vars;

    % Normalize the probablities for each observation vector.
    % This complete Expectation step.

    data.prob = sum(PY_x,2);
    data.prob = data.prob ./ sum(data.prob);

    % Maximisation step.
    for ngau = 1:Ngau
        % Obtain
        data.means(:, ngau)   = expect_mean(x, PY_x(ngau,:));
        [data.vars(:,:, ngau), like(ngau, :)]  = expect_var_gauss_dist(x,  PY_x(ngau,:), data.means(:,ngau), data.prob(ngau));
    end

    PY_x = like;
    [PY_x] = norm_prob(PY_x);

    sum_like = sum(like,1);
    loglike(iter) = sum(log(sum_like));
    if(iter>1)
        if(diff(loglike(end-1:end))/size(x,2) < diffperunitlike)
            %disp(sprintf('Converged in %d Iterations',iter));
            break;
        end
    end
end
%plot(loglike);hold on;
Tloglike = loglike(end);

function [v, y]  = expect_var_gauss_dist(x, px, m, mix_prior);
global var_floor;
[M N] = size(x);
m = repmat(m,1,N);
x_m = x-m;
% Extend px and take its root
sum_px = sum(px);
sq_px = repmat(px.^(1/2), M, 1);
sq_x_m = sq_px .* x_m;

v = sum(sq_x_m .* sq_x_m, 2)/sum_px;
v(v<var_floor) = var_floor;

y = mix_prior * gauss_dist_meanS(x_m, v);

function [m] = expect_mean(x,px)
sum_px = sum(px);
px = repmat(px, size(x,1),1);
m = sum(px.*x,2)/sum_px;

function [v, x_m] = expect_var(x,px,m)
[M N] = size(x);
m = repmat(m,1,N);
x_m = x-m;
% Extend px and take its root
sum_px = sum(px);
sq_px = repmat(px.^(1/2), M, 1);
sq_x_m = sq_px .* x_m;
v = sum(sq_x_m .* sq_x_m, 2)/sum_px;


function y = gauss_dist(x, m, v);
[M N] = size(x);
m = repmat(m,1, N);
x_m = x-m;
y = gauss_dist_meanS(x_m, v);


function y = gauss_dist_meanS(x_m, v);
global var_floor;
[M N] = size(x_m);
v(v<var_floor) = var_floor;
sq_det_v = sqrt(prod(v));

inv_v = 1./v;
temp_mult = repmat(inv_v, 1, N) .* x_m;
mahal_x   = sum(x_m .* temp_mult, 1);

y = exp(-mahal_x/2)/sq_det_v;
% Not evaluating normalising factor (2*pi..)as it is same for the different
% gaussians; it will anyway get equalised when normalising.


function [PY_x] = norm_prob(PY_x);
% Sum along the dimension of Ngau; sum out, divide.
norm_sum = sum(PY_x,1);

%% Ids which sum to 0 (but in general probability can never be truly 0)
Zid = find(norm_sum==0);
%% Ids which are ok (not 0)
okid = find(norm_sum~=0);

%% Must come after above steps.
norm_sum = repmat(norm_sum, size(PY_x, 1), 1);

% The below has some issues:
% What if norm_sum is 0.
PY_x(:, okid) = PY_x(:, okid) ./ norm_sum(:, okid);

% For zero probability cases, make them uniform..Is this right way?
PY_x(:, Zid)  = 1/size(PY_x,1);