function [] = speaker_id

% Top level file for speaker identification
% This package is for a closed set speaker identification (currently it is not speaker verification, 
% the package can be adapted for verification but it will require a UBM adaptation stage)
% The package is based on a Gaussian Mixture Model classifier

% The package is set up to operate on the standard 13-dimensional Sphinx 
% MFCC features. Currently C0 feature is not used for speaker identification

global mypath;  % This is the path for the directory where speaker_id.m file is located.

global mySesspath % Path for list of training and testing files
global myDatapath; % Path for storing intermediate data generated in this prgram, for faster processing and later reuse the program reads and saves the training/testing features.
global myTrainfeatspath myTestfeatpath; 
global trainlabel trainmodel;
global trainfiles testfiles;
global trainlist testlist;
global testlabel;
global var_floor Nmix;  

%% Specifiy all paths
%% It is required that all the directories exist
% Assign values to mypath, ..
% Example :
% mypath = 'C:\Speaker_ID\';
% Note that there is a '\' after '\Speaker_ID'
mypath = 'D:\Matlab_work\work\project\speech\Packages\SpeakerID_Package\V3\';
myTrainfeatspath = [mypath, 'Train\'];
myTestfeatpath = [mypath, 'Test\'];
mySesspath = [mypath, 'Sessions\'];
myDatapath = [mypath, 'Data\'];
trainlist = [mySesspath, 'train_simple.lst'];
testlist = [mySesspath, 'test_simple.lst'];
%% Specified all paths

%% Some Parameters in Training and Testing
var_floor = 1e-4; 
Nmix = 32;  % Number of Gaussian mixture models

%% All the required functions for Training and Testing
routines = strcat(mypath, 'Routines'); addpath(routines);

%%%% Begin Training %%%%%

trainlabel = '_train_Clean'; % A label to identify a particular training condition
trainmodel = strcat('train_model_cmn_noc0_nmix_', num2str(Nmix), trainlabel);

trainfiles = strcat(myDatapath, 'train_feats', trainlabel, '.mat');
%% Generate requisite files to train on
generate_loadfile_train_simple;

%% Generate models from the training files generated before.
get_gmm_model(Nmix);

%%%% End Training %%%% 

%%%% Begin Testing %%%%%
testlabel = '_test_Clean'; % A label to identify a particular testing condition
testfiles = strcat(myDatapath, 'test_feats', testlabel, '.mat');
generate_loadfile_test_simple;
Err = gmm_identification;
%%%% End Testing %%%%%

fprintf('\nSID Accuracy = %.2f\n', 1-Err);
%% Remove path
rmpath(routines);